﻿//Copyright (c) Microsoft Corporation.  All rights reserved.

using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Threading;
using Microsoft.WindowsAPICodePack.Shell;
using System.Windows.Input;
using Microsoft.WindowsAPICodePack.Shell.PropertySystem;
using System.Windows.Media;
using System.Windows.Interop;

namespace Microsoft.WindowsAPICodePack.Samples.SearchApp
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private int top = 10, left = 10;    // for glass
        private bool neverRendered = true;  // for glass
        private const int WM_DWMCOMPOSITIONCHANGED = 0x031E; // for glass (when DWM / glass setting is changed)
        internal ShellSearchFolder searchFolder = null;
        AdvancedSearch advWindow;   // keep only one instance of the advanced window (unless user closes it)

        int currentIndex = -1; // for our navigation history
        List<ShellContainer> navigationHistory = new List<ShellContainer>();

        public Window1()
        {
            InitializeComponent();
            DragThumb.DragDelta += OnMove;

            this.SourceInitialized += new EventHandler(Window1_SourceInitialized);
            this.Loaded += new RoutedEventHandler(Window1_Loaded);

            // Handle double clicks on the ListView items - 
            // for our search app, this is particularly useful if we have stacks showing up in the result
            // and the user wants to drill down.
            // Or if the search results contains folders...
            this.listView1.MouseDoubleClick += new System.Windows.Input.MouseButtonEventHandler(listView1_MouseDoubleClick);
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            HwndSource source = HwndSource.FromHwnd(new WindowInteropHelper(this).Handle);
            source.AddHook(new HwndSourceHook(WndProc));
        }

        void listView1_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            // When the user tries to navigate a ShellContainer,
            // change our ListView's ItemsSource to the new ShellContainer object.
            if (listView1.SelectedItem != null && listView1.SelectedItem is ShellContainer)
            {
                // Add to our navigation history
                navigationHistory.Add((ShellContainer)listView1.SelectedItem);
                currentIndex++;

                // Update the listview's itemsource
                listView1.ItemsSource = (ShellContainer)listView1.SelectedItem;

                if (listView1.Items.Count > 0)
                    listView1.SelectedIndex = 0;

                UpdateNavButtons();
            }
        }

        internal void Search()
        {
            List<ShellObject> items = new List<ShellObject>();
            foreach (ShellObject so in searchFolder)
                items.Add(so);

            // Update the listview's itemsource
            listView1.ItemsSource = items;

            if (listView1.Items.Count > 0)
                listView1.SelectedIndex = 0;

            // Add to our navigation history
            navigationHistory.Add(searchFolder);
            currentIndex++;

            UpdateNavButtons();
        }

        #region For the Aero glass effect

        private IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            // handle the message for DWM when the aero glass is turned on or off
            if (msg == WM_DWMCOMPOSITIONCHANGED)
            {
                if (GlassHelper.IsGlassEnabled)
                {
                    // Extend glass
                    Rect bounds = VisualTreeHelper.GetContentBounds(listView1);
                    GlassHelper.ExtendGlassFrame(this, new Thickness(bounds.Left, bounds.Top, bounds.Right, bounds.Bottom));
                }
                else
                {
                    // turn off glass...
                    GlassHelper.DisableGlassFrame(this);
                }

                handled = true;
            }

            return IntPtr.Zero;
        }

        private void OnMove(object s, DragDeltaEventArgs e)
        {
            left += (int)e.HorizontalChange;
            top += (int)e.VerticalChange;
            this.Left = left;
            this.Top = top;
        }

        void Window1_SourceInitialized(object sender, EventArgs e)
        {
            Rect bounds = VisualTreeHelper.GetContentBounds(listView1);
            GlassHelper.ExtendGlassFrame(this, new Thickness(bounds.Left, bounds.Top, bounds.Right, bounds.Bottom));
        }

        protected override void OnContentRendered(EventArgs e)
        {
            if (this.neverRendered)
            {
                // The window takes the size of its content because SizeToContent
                // is set to WidthAndHeight in the markup. We then allow
                // it to be set by the user, and have the content take the size
                // of the window.
                this.SizeToContent = SizeToContent.Manual;

                FrameworkElement root = this.Content as FrameworkElement;
                if (root != null)
                {
                    root.Width = double.NaN;
                    root.Height = double.NaN;
                }

                this.neverRendered = false;
            }

            base.OnContentRendered(e);
        }


        #endregion

        private void ButtonBack_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex > 0 && ((currentIndex - 1) > -1))
            {
                currentIndex--;

                // Update the listview's itemsource
                listView1.ItemsSource = navigationHistory[currentIndex];

                if (listView1.Items.Count > 0)
                    listView1.SelectedIndex = 0;
            }

            UpdateNavButtons();
        }

        private void ButtonForward_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex >= 0 && ((currentIndex + 1) < navigationHistory.Count))
            {
                currentIndex++;

                // Update the listview's itemsource
                listView1.ItemsSource = navigationHistory[currentIndex];

                if (listView1.Items.Count > 0)
                    listView1.SelectedIndex = 0;
            }

            UpdateNavButtons();
        }

        private void UpdateNavButtons()
        {
            if (currentIndex <= 0)
                ButtonBack.IsEnabled = false;
            else if (navigationHistory.Count > 1 && (currentIndex > 0))
                ButtonBack.IsEnabled = true;

            if (navigationHistory.Count > 1 && (currentIndex < navigationHistory.Count - 1))
                ButtonForward.IsEnabled = true;
            else
                ButtonForward.IsEnabled = false;
        }

        private void SearchTextBox_Search(object sender, RoutedEventArgs e)
        {
            // Search... on any letters typed
            if (!string.IsNullOrEmpty(SearchBox.Text))
            {
                SearchCondition combinedPropertyCondition = GetPropertyCondition(SearchBox.Text);

                SearchCondition dateCondition = SearchConditionFactory.CreateLeafCondition(
                    SystemProperties.System.Title, SearchBox.Text, SearchConditionOperation.ValueContains);

                SearchCondition finalSearchCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.Or,
                    false, combinedPropertyCondition, dateCondition);

                // Create a new search folder by setting our search condition and couple of search scopes

                if (ShellLibrary.IsPlatformSupported)
                {
                    searchFolder = new ShellSearchFolder(
                        finalSearchCondition,
                        (ShellContainer)KnownFolders.DocumentsLibrary,
                        (ShellContainer)KnownFolders.PicturesLibrary,
                        (ShellContainer)KnownFolders.VideosLibrary);
                }
                else
                {
                    searchFolder = new ShellSearchFolder(
                            finalSearchCondition,
                            (ShellContainer)KnownFolders.Documents,
                            (ShellContainer)KnownFolders.Pictures,
                            (ShellContainer)KnownFolders.Videos);
                }

                Search();
            }
            else
            {
                listView1.ItemsSource = null;   // nothing was typed, or user deleted the search query (clear the list).
                currentIndex++;
            }
        }

        private SearchCondition GetPropertyCondition(string searchText)
        {
            // Loop through each word user types and create a search condition
            string[] words = searchText.Split(' ');

            // This is what we'll return at the end
            SearchCondition combinedPropertyCondition = null;

            foreach (string word in words)
            {
                // Our first sample condition - compare the search query with the System.Filename
                SearchCondition propertyCondition1 = SearchConditionFactory.CreateLeafCondition(
                SystemProperties.System.FileName,
                word,
                SearchConditionOperation.ValueContains);

                // Second sample condition - compare the search query with the System.Keywords (tags)
                SearchCondition propertyCondition2 = SearchConditionFactory.CreateLeafCondition(
                    SystemProperties.System.Keywords,
                    word,
                    SearchConditionOperation.ValueContains);

                SearchCondition tmpCombinedCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.Or,
                    false, propertyCondition1, propertyCondition2);

                // If we already have an existing condition, do "AND" operation between these two
                if (combinedPropertyCondition != null)
                {
                    combinedPropertyCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.And,
                        false, combinedPropertyCondition, tmpCombinedCondition);
                }
                else
                    combinedPropertyCondition = tmpCombinedCondition;
            }

            return combinedPropertyCondition;
        }

        private void CommandBinding_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            SearchBox.Focus();
        }

        private void buttonSearchAdv_Click(object sender, RoutedEventArgs e)
        {
            if (advWindow == null)
            {
                advWindow = new AdvancedSearch();
                advWindow.MainWindow = this;
                advWindow.Closed += new EventHandler(advWindow_Closed);
            }

            if (!advWindow.IsVisible)
                advWindow.Show();
            else
            {
                advWindow.Visibility = Visibility.Visible;
                advWindow.Focus();
            }
        }

        void advWindow_Closed(object sender, EventArgs e)
        {
            advWindow = null;
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            if (advWindow != null)
                advWindow.Close();
        }
    }

    /// <summary>
    /// ImageView displays image files using themselves as their icons.
    /// In order to write our own visual tree of a view, we should override its
    /// DefaultStyleKey and ItemContainerDefaultKey. DefaultStyleKey specifies
    /// the style key of ListView; ItemContainerDefaultKey specifies the style
    /// key of ListViewItem.
    /// </summary>
    public class ImageView : ViewBase
    {
        #region DefaultStyleKey

        protected override object DefaultStyleKey
        {
            get { return new ComponentResourceKey(GetType(), "ImageView"); }
        }

        #endregion

        #region ItemContainerDefaultStyleKey

        protected override object ItemContainerDefaultStyleKey
        {
            get { return new ComponentResourceKey(GetType(), "ImageViewItem"); }
        }

        #endregion
    }

    public static class CustomCommands
    {
        public static RoutedCommand SearchCommand = new RoutedCommand("SearchCommand", typeof(CustomCommands));
    }
}
