﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing.Printing;
using System.Diagnostics;
using System.Drawing;

namespace PrintDemo
{
    class ProcessPrintDocument : PrintDocument
    {
        private Font _pageHeaderFont, _rowHeaderFont, _rowTextFont;
        float _pageHeaderHeight, _rowHeaderHeight, _rowHeight;

        bool _disposed = false;

        int _currentPage = 0;
        int _currentRow = 0;

        const int ColumnWidth = 150;

        private IList<Process> _processes;

        public ProcessPrintDocument(IList<Process> processes)
        {
            _processes = processes;

            _pageHeaderFont = new Font("Arial", 14.0f);
            _rowHeaderFont = new Font("Arial", 10.0f, FontStyle.Bold | FontStyle.Underline);
            _rowTextFont = new Font("Verdana", 10.0f);
        }

        protected override void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    _pageHeaderFont.Dispose();
                    _rowHeaderFont.Dispose();
                    _rowTextFont.Dispose();
                }
                base.Dispose(disposing);
            }
            _disposed = true;
        }

        protected override void OnBeginPrint(PrintEventArgs e)
        {
            base.OnBeginPrint(e);
            //do any initial setup here
        }

        protected override void OnEndPrint(PrintEventArgs e)
        { 
            base.OnEndPrint(e);
            //do any initial finish work here
        }

        protected override void OnPrintPage(PrintPageEventArgs e)
        {
            base.OnPrintPage(e);
            
            _pageHeaderHeight = e.Graphics.MeasureString("A", _pageHeaderFont).Height;
            _rowHeaderHeight = e.Graphics.MeasureString("A", _rowHeaderFont).Height;
            _rowHeight = e.Graphics.MeasureString("A", _rowTextFont).Height;

            _currentPage++;
            
            PrintPageHeader(e);

            PrintContent(e);

            PrintPageFooter(e);

            //this is important! You won't get more pages if you don't set this
            //Also important to set it to false at some point, so you don't print
            //forever!
            e.HasMorePages = (_currentRow < _processes.Count - 1);
        }

        private void PrintPageHeader(PrintPageEventArgs e)
        {
            //find where we should print our header
            Rectangle headerRect = new Rectangle(e.PageBounds.Left, e.PageBounds.Top,
                e.PageBounds.Right - e.PageBounds.Left,
                e.MarginBounds.Top - e.PageBounds.Top);
            if (headerRect.Height < 50)
            {
                headerRect.Height = 50;
            }
            StringFormat headerFormat = new StringFormat();
            headerFormat.LineAlignment = StringAlignment.Center;
            headerFormat.Alignment = StringAlignment.Center;

            string headerText = string.Format("Processes - page {0}", _currentPage);
            e.Graphics.DrawString(headerText, _pageHeaderFont, Brushes.Black, headerRect, headerFormat);
        }

        private void PrintPageFooter(PrintPageEventArgs e)
        {
            //let's draw an image just because we can
            PointF location = new PointF(e.MarginBounds.Left, e.MarginBounds.Bottom);
            int height = e.PageBounds.Bottom - e.MarginBounds.Bottom;
            RectangleF rect = new RectangleF(location, new Size (e.MarginBounds.Width, height));
            e.Graphics.DrawImage(Properties.Resources.SampleImage, rect);
        }

        private void PrintContent(PrintPageEventArgs e)
        {
            PointF currentLoc = new Point(e.MarginBounds.Left, e.MarginBounds.Top);

            PrintRowHeader(e, ref currentLoc);
            bool lastRow = false, lastRowOnPage = false;
            while (!lastRow && !lastRowOnPage)
            {
                currentLoc.Y += _rowHeight;
                currentLoc.X = e.MarginBounds.Left;
                lastRow = (_currentRow == _processes.Count - 1);
                lastRowOnPage = (currentLoc.Y + _rowHeight > e.MarginBounds.Bottom);
                if (currentLoc.Y + _rowHeight < e.MarginBounds.Bottom)
                {
                    PrintRow(e, _currentRow, ref currentLoc);
                    _currentRow++;
                }
            }
        }

        private void PrintRowHeader(PrintPageEventArgs e, ref PointF currentLoc)
        {
            string[] columnHeaders = { "Process", "PID", "Working Set", "Base Priority" };

            foreach (string header in columnHeaders)
            {
                RectangleF rect = new RectangleF(currentLoc.X, currentLoc.Y, ColumnWidth, _rowHeight);
                e.Graphics.DrawString(header, _rowHeaderFont, Brushes.Black, rect);
                currentLoc.X += ColumnWidth;
            }
        }

        private void PrintRow(PrintPageEventArgs e, int _currentRow, ref PointF currentLoc)
        {
            Process proc = _processes[_currentRow];
            RectangleF destRect = new RectangleF(currentLoc, new SizeF(ColumnWidth, _rowHeight));
            
            e.Graphics.DrawString(_currentRow.ToString()+" " + proc.ProcessName, _rowTextFont, Brushes.Black, destRect);
            destRect.Offset(ColumnWidth, 0);

            e.Graphics.DrawString(proc.Id.ToString(), _rowTextFont, Brushes.Black, destRect);
            destRect.Offset(ColumnWidth, 0);

            e.Graphics.DrawString(proc.WorkingSet64.ToString("N0"), _rowTextFont, Brushes.Black, destRect);
            destRect.Offset(ColumnWidth, 0);

            e.Graphics.DrawString(proc.BasePriority.ToString(), _rowTextFont, Brushes.Black, destRect);
            destRect.Offset(ColumnWidth, 0);
        }
    }
}
